MODULE ABB_FHRobotLib
! **********************************************************************
!              (C) Copyright OMRON CORPORATION 2018             
!                      All Rights Reserved           
! **********************************************************************
! File name    : ABB_FHRobotLib.mod         
! Description  : FH robot vision library
! **********************************************************************
    
!********* Global Parameters *********
    !Connection state to the server
    VAR bool bconnected := FALSE;

    !Tcp
    LOCAL CONST num max_retry_count := 99;
    LOCAL CONST num min_retry_count := 0;
    LOCAL CONST num max_time_out := 99;
    LOCAL CONST num min_time_out := 0;

    !CommandID
    LOCAL CONST num cmdid_echo := 1;
    LOCAL CONST num cmdid_set_para := 10;
    LOCAL CONST num cmdid_get_para := 11;
    LOCAL CONST num cmdid_cur_pos := 12;
    LOCAL CONST num cmdid_get_ver := 13;
    LOCAL CONST num cmdid_abs_move := 100; 
    
    !String
    VAR bool mod_flag := TRUE;
    LOCAL CONST num sendrecv_mod := 10000;
    LOCAL CONST string str_robot_cmd := "RCMD";
    LOCAL CONST string str_separate := " ";
    LOCAL CONST string str_delimiter := "\0D";
    LOCAL CONST num max_param_length := 12;
    CONST num max_param_num := 9;
    CONST num max_str_length := 128;

    !Scene No
    CONST num min_scene_no := 0;
    CONST num max_scene_no := 127;
    
    !Version
    LOCAL CONST string version := "1.00";
    
    !Status
    CONST num success := 0;
    CONST num err_scene_no := -1000;
    CONST num err_con_input := -1200;
    CONST num err_con_retry := -1202;
    CONST num err_no_connect := -1203;
    CONST num err_con_timeout := -1204;
    CONST num err_recv_input := -1300;
    CONST num err_recv := -1301;
    CONST num err_recv_maxstr := -1302;
    CONST num err_recv_retry := -1303;
    CONST num err_recv_timeout := -1304;
    CONST num err_decode_input := -1400;
    CONST num err_overlimit := -1500;
    CONST num err_cmd_id := -1501;
    CONST num err_cmd_format := -1502;
    CONST num err_rangeout := -1503;
    CONST num err_cmd_length := -1504;
    CONST num err_send := -1601;
    CONST num err_send_maxstr := -1602;
    CONST num err_trig := -1800;
    CONST num err_tjg := -1801;
    
    !Initialize Speed
    VAR speeddata tmp_speed := v100; 
   
!********************************************

! ***************************************************************************
! Title         : Connect to the Tcp Server
! Function name : fhconnect(fh_Socket, ipAddress, port_no, retries_connect, timeout_connect, err_no)
! Parameters    : socketdev fh_Socket;          Socket device
!                 string    inputIP;            Input IP address
!                 num       port_no;            Port No.
!                 num       retries_connect;    Tetries count
!                 num       timeout_connect;    Timeout
! Return value  : num       err_no;             Error No.
! ***************************************************************************    
PROC fhconnect(VAR socketdev fh_socket, string ip_address, num port_no, num retries_connect, num timeout_connect, VAR num err_no)

    VAR num countRetries := 0;

    IF bconnected = FALSE THEN
        ! Check the range of input parameter
        IF (retries_connect < min_retry_count) OR (retries_connect > max_retry_count) THEN
            err_no := err_con_input;
            TPWrite ("ERROR:fhconnect():RETRY:")\Num:=err_no;
            RETURN;
        ENDIF
        IF (timeout_connect < min_time_out) OR (timeout_connect > max_time_out) THEN
            err_no := err_con_input;
            TPWrite ("ERROR:fhconnect():TIMEOUT:")\Num:=err_no;
            RETURN;
        ENDIF
         
        !Set Timeout Parameter
        IF timeout_connect = 0 THEN
            timeout_connect := WAIT_MAX;
        ENDIF
            
        ! Connect to Tcp server
        WaitTime 2;
        SocketClose fh_socket;
        SocketCreate fh_socket;
        SocketConnect fh_socket, ip_address, port_no, \Time:=timeout_connect;
        
        ! Change Status of Connect
        bconnected := TRUE;
        
        !Change Status of ErrNo
        err_no := success;
    ENDIF

    !Error Check
    ERROR
    IF ERRNO = ERR_SOCK_TIMEOUT THEN
        err_no := err_con_timeout;
        TPWrite ("ERROR:fhconnect():TIMEOUT:")\Num:=err_no;     
        countRetries := countRetries + 1;
        IF countRetries < retries_connect THEN
            RETRY;     
        ELSE
            err_no := err_con_retry;
            TPWrite ("ERROR:fhconnect():RETRY:")\Num:=err_no;
            RETURN;
        ENDIF
    ENDIF

ENDPROC

! ***************************************************************************
! Title         : Socket Close
! Function name : fhclose(fh_socket,err_no)
! Parameters    : socketdev fh_socket;           Socket device
! Return value  : num       err_no;              Error No.  
! ***************************************************************************    
PROC fhclose(VAR socketdev fh_socket,VAR num err_no)
    
    IF bconnected = TRUE THEN
        WaitTime 2;
        ! Socket close
        SocketClose fh_socket;
        ! Change Status of Connect
        bconnected := FALSE;
        !Change Status of ErrNo
        err_no := success;
    ENDIF   
    
ENDPROC

! ***************************************************************************
! Title         : Receive String from Tcp Server
! Function name : fhrecvstring(fh_socket, retries_recv, timeout_recv, recv_string{*}, err_no)
! Parameters    : socketdev fh_socket;          Socket device
!                 num       retries_recv;       Retries count
!                 num       timeout_recv;       Timeout
! Return value  : string    recv_string{*};     response string
!                 num       err_no;             Error No.
! ***************************************************************************
PROC fhrecvstring(VAR socketdev fh_socket, num retries_recv, num timeout_recv, VAR string recv_string{*}, VAR num err_no)

    VAR string strPart := "";
    VAR num countretries := 0;
    VAR num recv_strlen := 1;
    VAR bool exit_flag := FALSE;
    
    !Initialize
    FOR i FROM 1 TO max_str_length DO
        recv_string{i} := "";
    ENDFOR  
    
    ! Check the range of input parameter
    IF (retries_recv < min_retry_count) OR (retries_recv > max_retry_count) THEN
        err_no := err_recv_input;
        TPWrite ("ERROR:fhrecvstring():RETRY:")\Num:=err_no;
        RETURN;
    ENDIF
    IF (timeout_recv < min_time_out OR timeout_recv > max_time_out) THEN
        err_no := err_recv_input;
        TPWrite ("ERROR:fhrecvstring():TIMEOUT:")\Num:=err_no;
        RETURN;
    ENDIF

    IF bconnected = TRUE THEN
        !Set Timeout Parameter
        IF timeout_recv = 0 THEN
            timeout_recv := WAIT_MAX;
        ENDIF
 
        ! Receive String from Tcp Server
        WHILE exit_flag = FALSE DO
            SocketReceive fh_socket, \Str:=strPart,\ReadNoOfBytes:=1, \Time:=timeout_recv;
            IF (strPart = STR_DELIMITER) OR (recv_strlen > max_str_length) THEN
                exit_flag := TRUE;
            ELSE
                recv_string{recv_strlen} :=strPart;
                Incr recv_strlen; 
            ENDIF
            
        ENDWHILE
      
        !Stirng Length Check
        IF recv_strlen = 1 THEN
            err_no := err_recv;
            TPWrite ("ERROR:fhrecvstring():NO_DATA:")\Num:=err_no;
            RETURN;
        ELSEIF recv_strlen > max_str_length THEN
            err_no := err_recv_maxstr;
            TPWrite ("ERROR:fhrecvstring():STRING_LEN:")\Num:=err_no;
            RETURN;
        ENDIF
    ELSE
        err_no := err_recv;
        TPWrite ("ERROR:fhrecvstring():NO_CONNECTION:")\Num:=err_no;
        RETURN;       
    ENDIF

    !Change Status of ErrNo
    err_no := success;
    
    !Error Check
    ERROR
    IF ERRNO = ERR_SOCK_TIMEOUT THEN
        err_no := err_recv_timeout;
        countRetries := countRetries + 1;
        TPWrite ("ERROR:fhrecvstring():TIMEOUT:")\Num:=err_no;        
        IF countRetries < retries_recv THEN
            RETRY;
        ELSE
            err_no := err_recv_retry;
            TPWrite ("ERROR:fhrecvstring():RETRY:")\Num:=err_no;
            RETURN;
        ENDIF
    ELSEIF ERRNO = ERR_SOCK_CLOSED THEN 
        err_no := err_recv;
        TPWrite ("ERROR:fhrecvstring():CLOSE_SOCKET:")\Num:=err_no;
        RETURN;
    ENDIF
    
ENDPROC

! ***************************************************************************
! Title         : Send String to Tcp Server
! Function name : fhsendstring(fh_socket, send_string{*}, err_no)
! Parameters    : socketdev fh_socket;          Socket device
!                 string    send_string{*};     Response string
! Return value  : num       err_no;             Error No.
! ***************************************************************************    
PROC fhsendstring(VAR socketdev fh_socket, string send_string{*}, VAR num err_no)
    
    VAR num msg_num := 0;
    VAR num send_strlen := 0;
    VAR num param_count := 0;
    VAR num i := 0;
    VAR string tmp_string := "";
    VAR rawbytes send_bytes;
    
    !Initialize
    ClearRawBytes send_bytes;
    
    !Param Number Calculation
    msg_num :=  1 + max_param_num; 
    
    !Param Count Check
    FOR i FROM 1 TO msg_num DO
        send_strlen := send_strlen + StrLen(send_string{i}); 
        IF StrLen(send_string{i}) > 0 THEN
            Incr param_count;
        ENDIF   
    ENDFOR
    
    !String Length Check
    IF send_strlen = 0 THEN
        err_no := err_send;
        TPWrite ("ERROR:fhsendstring():NO_DATA:")\Num:=err_no;
        RETURN;
    ELSEIF send_strlen + param_count  > max_str_length THEN
        err_no := err_send_maxstr;
        TPWrite ("ERROR:fhsendstring():STRING_LEN:")\Num:=err_no;
        RETURN;
    ENDIF
    
    !Send String
    IF bconnected = TRUE THEN
        FOR i FROM 1 TO param_count DO
            tmp_string := send_string{i};
            IF i <> param_count THEN
                tmp_string:= tmp_string + str_separate;
            ELSE
                tmp_string:= tmp_string + str_delimiter;
            ENDIF
            PackRawBytes tmp_string, send_bytes, (RawBytesLen(send_bytes)+1) \ASCII;
        ENDFOR
        SocketSend fh_socket,\RawData:=send_bytes,\NoOfBytes:=RawBytesLen(send_bytes);
    ELSE

        err_no := err_send;
        TPWrite ("ERROR:fhsendstring():NO_CONNECTION:")\Num:=err_no;
        RETURN;
    ENDIF
    
    !Change Status of ErrNo
    err_no := success;

    !Error Check    
    ERROR
    IF ERRNO = ERR_SOCK_CLOSED THEN 
        err_no := err_send;
        TPWrite ("ERROR:fhsendstring():CLOSE_SOCKET:")\Num:=err_no;
        RETURN;
    ENDIF

ENDPROC

! ***************************************************************************
! Title         : Decode Command String
! Function name : fhdecode(command_string{*}, cmdparam{*}, paracount, err_no) 
! Parameters    : string    command_string{*}   command string
! Return value  : num       cmdparam{*}         command parameter
!                 num       paracount           parameter count
!                 num       err_no;             Error No.
! ***************************************************************************        
PROC fhdecode(string command_string{*}, VAR num cmdparam{*}, VAR num paracount, VAR num err_no)  
      
    VAR num msg_num := 0;
    VAR bool exit_flag := FALSE;
    VAR num command_strlen := 0;
    VAR num space_pos{max_str_length};
    VAR string strcmdPara{10};
    VAR string tmp_string;
    VAR num tmplen;
    VAR num dotpos;
    VAR num startindex := 0;
    VAR bool convert_judge;
    VAR num tmpcount := 0;
    VAR num i := 0;
    VAR num j := 1;
    
    !Initialize 
    paracount := 0;
    
    !Param Number Calculation
    msg_num :=  1 + max_param_num; 
    
    !Check Command Stribg Length
    WHILE exit_flag = FALSE DO
        IF (command_string{command_strlen + 1} =  "") OR (command_strlen = max_str_length) THEN
            exit_flag := TRUE;
        ELSE
            Incr command_strlen;
        ENDIF 
    ENDWHILE
    
    IF (command_strlen > 0) AND (command_strlen < max_str_length) THEN
        !Get Space Point
        FOR i FROM 1 TO command_strlen  DO
             IF command_string{i} = str_separate THEN
                 space_pos{j} := i;
                 Incr j;
             ENDIF
        ENDFOR
        space_pos{j} := command_strlen +1;
        
        !Get Command Header
        IF space_pos{1} <> 1 THEN
            FOR i FROM 1 TO space_pos{1} - 1 DO
                strcmdPara{1} := strcmdPara{1} + command_string{i};
                startindex := 2;
            ENDFOR
        ELSE
             FOR i FROM space_pos{1} + 1 TO space_pos{2} - 1 DO
                strcmdPara{1} := strcmdPara{1} + command_string{i};
                startindex := 3;
            ENDFOR
        ENDIF
        
        !Check Command Header
        IF strMatch(strcmdPara{1},1,str_robot_cmd) <> 1 THEN
            err_no := err_cmd_format;
            TPWrite ("ERROR:fhdecode():NO_HEADER:")\num := err_no;
            RETURN;
        ENDIF
        
        !Get Parameter
        FOR i FROM startindex TO j STEP 1 DO
            IF space_pos{i} - space_pos{i -1}  <> 1 THEN
                FOR i FROM space_pos{i-1} + 1 TO space_pos{i} - 1 STEP 1 DO
                    tmp_string := tmp_string + command_string{i};
                ENDFOR
                tmplen :=StrLen(tmp_string);
                IF tmplen > max_param_length THEN
                    err_no := err_cmd_format;
                    TPWrite ("ERROR:fhdecode():PARAM_TOO_LONG:")\num := err_no;
                    RETURN; 
                ENDIF
                dotpos := StrFind(tmp_string,1,".");
                IF (dotpos > 0) AND ((tmplen - dotpos) >= 4) THEN 
                         tmp_string := strPart(tmp_string,1,dotpos + 3);
                ENDIF 
                Incr tmpcount;
                convert_judge := StrToVal(tmp_string,cmdParam{tmpcount});
                IF convert_judge THEN
                    Incr paracount;
                    !Parameter Number Check
                    IF paracount = max_param_num THEN
                        !Loop End
                        GOTO loop_end;
                    ENDIF
                ENDIF
                tmp_string := "";
            ENDIF 
        ENDFOR
    
     !Decode Loop End Label
     loop_end:       
        
    !Check Param Num Check
    IF paracount < 1 THEN
        err_no := err_cmd_format;
        TPWrite ("ERROR:fhdecode():PARAM_NUM:")\num := err_no;
        RETURN;
    ENDIF
    !Check CommandLength Check
    ELSE
        err_no := err_cmd_length;
        TPWrite ("ERROR:fhdecode():NO_CMD:")\num := err_no;
        RETURN;
    ENDIF
    
    !Change Status of ErrNo
    err_no := success;
    
ENDPROC

! ***************************************************************************
! Title         : Command Implement
! Function name : fhsetup(fh_socket, ip_address, port_no, retries_connect, timeout_connect, retries_recv, timeout_recv, err_no)
! Parameters    : socketdev fh_socket;           Socket device
!                 string    ip_address;         IP address
!                 string    port_no;            Port No.
!                 num       retries_connect;    Tetries count
!                 num       timeout_connect;    Timeout
!                 num       retries_recv;       Retries count
!                 num       timeout_recv;       Timeout
!                 num       err_no;             Error No.
! Return value  :
! ***************************************************************************  
PROC fhsetup(VAR socketdev fh_socket, string ip_address, num port_no, num retries_connect, num timeout_connect, num retries_recv, num timeout_recv, VAR num err_no)
    
    VAR num paracount := 0;
    VAR num cmdID := 0;
    var num cmdparam{max_param_num};
    VAR string recv_string{max_str_length};
    VAR string send_string{max_param_num + 1};
    
    ! Check Status of Connect
    IF bconnected = TRUE THEN
        fhclose fh_Socket,err_no; 
    ENDIF
    
    ! Connect
    WHILE bconnected = FALSE DO
        fhconnect fh_Socket, ip_address, port_no, retries_connect, timeout_connect, err_no;
        IF err_no <> success THEN
            GOTO exit_func;
        ENDIF
    ENDWHILE
    
    WHILE bconnected = TRUE DO
        !Initialize
        paraCount := 0;
        err_no := 0;
        FOR i FROM 1 TO max_str_length DO
            recv_string{i} := "";
        ENDFOR
        
        FOR i FROM 1 TO max_param_num + 1 DO
            send_string{i} := "";
        ENDFOR
        
        FOR i FROM 1 TO max_param_num DO
            cmdparam{i} := 0;
        ENDFOR
        
        
        !Recv String
        fhrecvstring fh_socket, retries_recv, timeout_recv, recv_string, err_no;
        
        !Error Check
        IF err_no <> success THEN
            GOTO exit_func;
        ENDIF
        
        !Decode Command String
        fhdecode recv_string, cmdparam, paracount, err_no;
        
        !Error Check
        IF err_no <> success THEN
            GOTO exit_func;
        ENDIF            
        
        IF paracount > 0 THEN
            IF err_no = success THEN
                cmdID := cmdparam{1};
                TEST cmdID
                CASE cmdid_echo:        ! Commnand Echo
                    fhecho cmdparam, paracount, send_string, err_no;
                CASE cmdid_set_para:    ! Set Parameter
                    fhsetrobotpara cmdparam, paracount, send_string, err_no;
                CASE cmdid_get_para:    ! Get Parameter
                    fhgetrobotpara cmdparam, paracount, send_string, err_no;
                CASE cmdid_cur_pos:     ! Get Current Position
                    fhgetcurpos cmdparam, paracount, mod_flag, send_string, err_no;
                CASE cmdid_get_ver:     ! Get Version
                    fhgetverinfo cmdparam, paracount, send_string, err_no;
                CASE cmdid_abs_move:    ! Move to Specified Position
                    fhabsmove cmdparam, paracount , mod_flag, send_string, err_no;
                DEFAULT:
                    err_no := err_cmd_id;
                    TPWrite ("ERROR:fhsetup():UNDEFINED_ID:")\Num:= err_no;
                    
                    !Set Response String
                    send_string{1} := str_robot_cmd;
                    send_string{2} := ValtoStr(cmdID);
                    send_string{3} := ValToStr(err_no);
                ENDTEST
            ENDIF
            
            ! Send Response String
            fhsendstring fh_socket, send_string, err_no;
            
            !Error Check
            IF err_no <> success THEN
                GOTO exit_func;
            ENDIF 
            
        ENDIF
        
    ENDWHILE
    
    !Close
    exit_func:
        fhclose fh_socket,err_no;
        
ENDPROC


! ***************************************************************************
! Title         : Command Echo
! Function name : fhecho(cmdParam{*}, paracount, echo_string{*}, err_no)
! Parameters    : num       cmdparam{*};    command parameter
!                 num       paracount;      parameter count
! Return value  : string    echo_string{*}; response string
!                 num       err_no;         Error No.
! ***************************************************************************
PROC fhecho(num cmdparam{*}, num paraCount, VAR string echo_string{*}, VAR num err_no)
    
    VAR num cmdID := 0;
    VAR string result;
    
    !Check Parameter Count
    IF (cmdparam{1} = cmdid_echo) AND (paraCount = 1) THEN
        err_no := success;
    ELSE
        err_no := ERR_CMD_FORMAT;
        TPWrite ("ERROR:fhecho():PARAM_NUM:")\Num:= err_no;
    ENDIF

    ! Set Response String
    echo_string{1} := str_robot_cmd;
    echo_string{2} := ValToStr(cmdparam{1});
    echo_string{3} := ValToStr(err_no);
    
ENDPROC

! ***************************************************************************
! Title         : Set Robot Parameter
! Function name : fhsetrobotpara(cmdparam{*}, paracount, res_string{*}, err_no)
! Parameters    : num       cmdpara{*};     command parameter
!                 num       paracount;      parameter count
! Return value  : string    res_string{*};  response string
!                 num       err_no;         Error No.
! ***************************************************************************    
PROC fhsetrobotpara(num cmdparam{*}, num paracount, VAR string res_string{*}, VAR num err_no)
    
    VAR num nspeed;
    
    !Initialize data
    nspeed := 0;
    !Check Parameter Count    
    IF (cmdparam{1} = cmdid_set_para) AND  (paracount = 2) THEN
        nspeed := cmdparam{2};
        !Check speed
        IF (nspeed < 1) OR (nspeed > 100) THEN
            err_no := err_overlimit;
            TPWrite ("ERROR:fhsetrobotpara():SPEED:")\Num:= err_no;
        ELSE
            nspeed := Round(nspeed);
            !Set Speed data
            err_no := success;
            SpeedRefresh nspeed;
        ENDIF
        
    ELSE
        !Check Paramter
        err_no := err_cmd_format;
        TPWrite ("ERROR:fhsetrobotpara():PARAM_NUM:")\Num:= err_no;           
    ENDIF
    
    ! Set response string
    res_string{1} := str_robot_cmd;
    res_string{2} := ValToStr(cmdparam{1});
    res_string{3} := ValToStr(err_no);
    
ENDPROC

! ***************************************************************************
! Title         : Get Robot Parameter
! Function name : fhgetrobotpara(cmdparam{*}, paracount, res_string{*}, err_no)
! Parameters    : num       cmdparam{*};    command parameter
!                 num       paracount;      parameter count
! Return value  : String    res_string{*};  response string
!                 num       err_no;         Error No.
! ***************************************************************************    
PROC fhgetrobotpara(num cmdparam{*}, num paracount, VAR string res_string{*}, VAR num err_no)
    
    VAR num nspeed;
    
    !Initialize
    nspeed := 0;
    
    !Check Parameter Count
    IF (cmdparam{1} = cmdid_get_para) AND (paracount = 1) THEN
        ! Get Parameter
        nspeed := CSpeedOverride(\CTask);
        err_no := success;
    ELSE
        nspeed := 0;
        err_no := err_cmd_format;
        TPWrite ("ERROR:fhgetrobotpara():PARAM_NUM:")\Num:= err_no;     
    ENDIF
    
    ! Set Response String
    res_string{1} := str_robot_cmd;
    res_string{2} := ValToStr(cmdparam{1});
    res_string{3} := ValToStr(err_no);
    res_string{4} := ValToStr(nspeed);
    
ENDPROC

! ***************************************************************************
! Title         : Get Current Position
! Function name : fhgetcurpos(cmdparam{*}, paracount, intflg ,res_string{*}, err_no)
! Parameters    : num       cmdparam{*};    command parameter
!                 num       paracount;      parameter count
!                 bool       intflag;       param choice FALSE:Real TRUE:Real*10000
! Return value  : string    res_string{*};  response string
!                 num       err_no;         Error No.
! ***************************************************************************     
PROC fhgetcurpos(num cmdparam{*}, num paracount, bool intflg,VAR string res_string{*}, VAR num err_no)
    
    VAR robtarget curTarget;
    CONST num corcount := 6;
    VAR num tmppos{corcount};
    VAR num i := 1;
    
    !Check Parameter Count
    IF (cmdparam{1} = cmdid_cur_pos) AND (paracount = 1) THEN

        ! Get present position
        curTarget := CRobT();
        tmppos{1} := Round((curTarget.trans.x)\Dec := 3);
        tmppos{2} := Round((curTarget.trans.y)\Dec := 3);
        tmppos{3} := Round((curTarget.trans.z)\Dec := 3);
        tmppos{4} := Round((EulerZYX(\X, curTarget.rot))\Dec := 3);
        tmppos{5} := Round((EulerZYX(\Y, curTarget.rot))\Dec := 3);
        tmppos{6} := Round((EulerZYX(\Z, curTarget.rot))\Dec := 3);
        
        IF intflg = TRUE THEN     
            tmppos{1} := tmppos{1} * sendrecv_mod;
            tmppos{2} := tmppos{2} * sendrecv_mod;
            tmppos{3} := tmppos{3} * sendrecv_mod;
            tmppos{4} := tmppos{4} * sendrecv_mod;
            tmppos{5} := tmppos{5} * sendrecv_mod;
            tmppos{6} := tmppos{6} * sendrecv_mod;
       ENDIF
       
        err_no := success;
    ELSE
        err_no := err_cmd_format;
        TPWrite ("ERROR:fhgetcurpos():PARAM_NUM:")\Num:= err_no;
    ENDIF
    
    IF err_no <> success THEN
        FOR i FROM 1 TO corcount DO
            tmppos{i} := 0;
        ENDFOR
    ENDIF
       
    ! Set Response String
    res_string{1} := str_robot_cmd;
    res_string{2} := ValToStr(cmdparam{1});
    res_string{3} := ValToStr(err_no);
    res_string{4} := ValToStr(tmppos{1});
    res_string{5} := ValToStr(tmppos{2});        
    res_string{6} := ValToStr(tmppos{3});
    res_string{7} := ValToStr(tmppos{4});
    res_string{8} := ValToStr(tmppos{5});
    res_string{9} := ValToStr(tmppos{6});
    
ENDPROC

! ***************************************************************************
! Title         : Get Version Information
! Function name : fhgetverinfo(cmdparam{*}, paramcount, res_string{*} err_no)
! Parameters    : num       cmdparam{*};    command parameter
!               : num       paracount;      parameter count
! Return value  : string    res_string{*};  response string
!               : num       err_no;         Error No.
! ***************************************************************************
Proc fhgetverinfo(num cmdparam{*}, num paracount, VAR string res_string{*}, VAR num err_no)

    VAR string local_ver; 
    VAR bool ok;

    !Initialize
    local_ver := "0.00";

    ! Check parameter count
    IF (cmdparam{1} = cmdid_get_ver) AND (paracount = 1) THEN
        err_no := success;
        local_ver := version;
    ELSE
        err_no := err_cmd_format;
        local_ver := "0.00";
        TPWrite ("ERROR:fhgetverinfo():PARAM_NUM:")\Num:= err_no;
    ENDIF

    ! Set Response String
    res_string{1} := str_robot_cmd;
    res_string{2} := ValToStr(cmdparam{1});
    res_string{3} := ValToStr(err_no);
    res_string{4} := local_ver;
    
ENDPROC
    
! ***************************************************************************
! Title         : Move to Specified Position
! Function name : fhabsmove(cmdparam{*}, paracount, intflg, res_string{*}, err_no)
! Parameters    : num       cmdparam{*};     command parameter
!                 num       paracount;      parameter count
!                 bool      intflg;         param choice FALSE:Real TRUE:Real*10000                     
! Return value  : String    res_string{*};   response string
!                 num       err_no;         Error No.
! ***************************************************************************    
PROC fhabsmove(num cmdparam{*}, num paracount, bool intflg, VAR string res_string{*}, VAR num err_no)
    
    VAR robtarget desttarget;
    VAR robtarget curtarget;
    VAR jointtarget jointpos;
    VAR num x;
    VAR num y;
    VAR num z;
    VAR num rx;
    VAR num ry;
    VAR num rz;
    
    IF (cmdparam{1} = cmdid_abs_move) AND (paracount = 7) THEN
        x := cmdparam{2};
        y := cmdparam{3};
        z := cmdparam{4};
        rx := cmdparam{5};
        ry := cmdparam{6};
        rz := cmdparam{7};
        
        IF intflg = TRUE  THEN
            x:= x / sendrecv_mod;
            y:= y / sendrecv_mod;
            z:= z / sendrecv_mod;
            rx:= rx / sendrecv_mod;
            ry:= ry / sendrecv_mod;
            rz:= rz / sendrecv_mod; 
        ENDIF
        
        ! Get Target Position
        desttarget := CRobT();
        desttarget.trans.x := x;
        desttarget.trans.y := y;
        desttarget.trans.z := z;
        desttarget.rot := OrientZYX(rz,ry,rx);

        jointpos := CalcJointT(desttarget, tool0, \WObj:=wobj0);
        
        ConfJ \On;
        WaitUntil \InPos, TRUE;
        MoveJ desttarget, tmp_speed, fine, tool0, \WObj:= wobj0;
        
        err_no := success;
        
    ELSE
        err_no := err_cmd_format;
        TPWrite ("ERROR:fhabsmove():PARAM_NUM:")\Num:= err_no;
    ENDIF
    
    ! Get Present Position
    IF err_no = success THEN
        curTarget := CRobT();
        x := Round((curTarget.trans.x)\Dec := 3);
        y := Round((curTarget.trans.y)\Dec := 3);
        z := Round((curTarget.trans.z)\Dec := 3);
        rx := Round((EulerZYX(\X, curTarget.rot))\Dec := 3);
        ry := Round((EulerZYX(\Y, curTarget.rot))\Dec := 3);
        rz := Round((EulerZYX(\Z, curTarget.rot))\Dec := 3);
        
        IF intflg = TRUE  THEN
            x:= x * sendrecv_mod;
            y:= y * sendrecv_mod;
            z:= z * sendrecv_mod;
            rx:= rx * sendrecv_mod;
            ry:= ry * sendrecv_mod;
            rz:= rz * sendrecv_mod; 
        ENDIF
        
    ELSE
        x := 0;
        y := 0;
        z := 0;
        rx := 0;
        ry := 0;
        rz := 0;
    ENDIF
    
    ! Set Response String
    res_string{1} := str_robot_cmd;
    res_string{2} := ValToStr(cmdparam{1});
    res_string{3} := ValToStr(err_no);
    res_string{4} := ValToStr(x);
    res_string{5} := ValToStr(y);        
    res_string{6} := ValToStr(z);
    res_string{7} := ValToStr(rx);
    res_string{8} := ValToStr(ry);
    res_string{9} := ValToStr(rz);
    
    ERROR
    IF ERRNO = ERR_OUTSIDE_REACH THEN
        err_no := err_rangeout;
        TPWrite ("ERROR:fhabsmove():OUT_RANGE:")\Num:= err_no;
        x := 0;
        y := 0;
        z := 0;
        rx := 0;
        ry := 0;
        rz := 0;
        ! Set Response String
        res_string{1} := str_robot_cmd;
        res_string{2} := ValToStr(cmdparam{1});
        res_string{3} := ValToStr(err_no);
        res_string{4} := ValToStr(x);
        res_string{5} := ValToStr(y);        
        res_string{6} := ValToStr(z);
        res_string{7} := ValToStr(rx);
        res_string{8} := ValToStr(ry);
        res_string{9} := ValToStr(rz);
        RETURN;
    ENDIF

    
ENDPROC

! ***************************************************************************
! Title         : Get Measured Position
! Function name : fhmeasureresult(retries_recv, timeout_recv, fh_socket, param[], err_no)
! Parameters    : num       retries_recv;   Receive retry count
!               : num       timeout_recv;   Receive wait time
!               : socketdev fh_socket       Socket_no
! Return value  : num       param{*};        Measured position
!               : num       err_no;         Error No.
! ***************************************************************************
PROC fhmeasureresult(num retries_recv, num timeout_recv, VAR socketdev fh_socket, VAR num param{*},VAR num err_no)
    
    VAR num paracount := 0;
    VAR num rcmd_length := 0;
    var num cmdparam{max_param_num};
    VAR string recv_string{max_str_length};
    VAR string tmp_string{max_str_length};
    VAR string send_string{max_param_num + 1};
    
    !Initialize
    paraCount := 0;
    err_no := 0;
    FOR i FROM 1 TO max_str_length DO
        recv_string{i} := "";
    ENDFOR
    
    FOR i FROM 1 TO max_param_num + 1 DO
        send_string{i} := "";
    ENDFOR
    
    FOR i FROM 1 TO max_param_num DO
        cmdparam{i} := 0;
    ENDFOR
    
    IF bconnected = TRUE THEN
        !Receive String
        fhrecvstring fh_socket, retries_recv, timeout_recv, recv_string, err_no;
        
        IF err_no <> success THEN
            RETURN;
        ENDIF
        
        !Receive String Check
        IF ((recv_string{1} = "O") AND (recv_string{2} = "K")) <> TRUE THEN
            err_no := err_trig;
            TPWrite ("ERROR:fhmeasureresult():TRIG_NG:")\Num:= err_no;
            RETURN;
        ENDIF
        
        !Initialize
        FOR i FROM 1 TO max_str_length DO
            recv_string{i} := "";
        ENDFOR
        
        !Recive String
        fhrecvstring fh_socket, retries_recv, timeout_recv, tmp_string, err_no;
        
        !Add Command Header
        rcmd_length := StrLen(str_robot_cmd);
        FOR i FROM 1 TO rcmd_length DO          
            recv_string{i} := StrPart(str_robot_cmd,i,1);
        ENDFOR
        recv_string{rcmd_length + 1} := str_separate;
        FOR i FROM 1 TO max_str_length - rcmd_length - 1 DO
            recv_string{i + rcmd_length +1} := tmp_string{i};
        ENDFOR

        !Decode Command String
        fhdecode recv_string, cmdparam, paracount, err_no;
        
        !Format Check
        IF paracount <> 7 THEN
            err_no := err_cmd_format;
            TPWrite ("ERROR:fhmeasureresult():PARAM_NUM:")\Num:= err_no;
            RETURN;
        ENDIF
        
        !Total Judge Check
        IF cmdparam{1} <> 1 THEN
            err_no := err_tjg;
            TPWrite ("ERROR:fhmeasureresult():TJG_NG:")\Num:= err_no;
            RETURN;
        ENDIF
        
        !Prametet Setting
        FOR i FROM 1 TO paracount -1 DO
            param{i} := cmdparam{i+1};
        ENDFOR
        
        err_no := success;
    ELSE
        err_no := err_no_connect;
        TPWrite ("ERROR:fhmeasureresult():NO_CONNECT:")\Num:= err_no;
        RETURN;
    ENDIF
    
ENDPROC
        
ENDMODULE