MODULE ABB_FHRobotSample
! **********************************************************************
!              (C) Copyright OMRON CORPORATION 2018             
!                      All Rights Reserved           
! **********************************************************************
! File name    : ABB_FHRobotSample.mod         
! Description  : FH robot vision Sample
! **********************************************************************

! ***************************************************************************
! Title         : FH Sample Main
! Function name : fhsample_main()
! Parameters    :
! Return value  :
! Date           Version         Details
! ***************************************************************************
PROC fhsample_main()
!;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
! 1. Example: Initialization sequence
! (1) Declare local variables
! (2) Initialize global variables
!
! 2. Example: Network connection sequence
! (1) Set the network configuration
! (2) Connect to the FH server
!
! 3. Example: Processing sequence of an application
! (1) Change the scene of the FH!
! (2) Move the robot to the imaging position
! (3) Register the current position of the robot to the FH before measurement
! (4) Execute measurement and get the measurement results of the FH
! (5) Move the robot to the approach position
! (6) Move the robot to the measured position
!
! 4. Example: Finalization sequence
! (1) Disconnect to the FH server
!;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! 1. Example: Initialization sequence
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (1)Declare local variables
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;    
    VAR socketdev fh_socket;            !Socket
    VAR string ip_address;              !IP address
    VAR num port_no;                    !Port No
    VAR num retries_connect;            !Connect Retry Count
    VAR num timeout_connect;            !Connect Wait Time
    VAR num retries_recv;               !Recieve Retry Count    
    VAR num timeout_recv;               !Recieve Wait Time
    VAR num err_no;                     !Error No. Return Value. Noneed to set
    
    VAR num scene_no;
    
    VAR num std_img_pos_x;
    VAR num std_img_pos_y;
    VAR num std_img_pos_z;
    VAR num std_img_pos_w;
    VAR num std_img_pos_p;
    VAR num std_img_pos_r;
    
    VAR num res_cmd_pos_x;
    VAR num res_cmd_pos_y;
    VAR num res_cmd_pos_z;
    VAR num res_cmd_pos_w;
    VAR num res_cmd_pos_p;
    VAR num res_cmd_pos_r;
    
    VAR num res_appro_pos_x;
    VAR num res_appro_pos_y;
    VAR num res_appro_pos_z;
    VAR num res_appro_pos_w;
    VAR num res_appro_pos_p;
    VAR num res_appro_pos_r;
    
    VAR num std_appro_dist;
    VAR num param{6};
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (2)Initialaize global variables
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    err_no := success;
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! 2. Example: Network connection sequence
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (1)Set the network configuration
    !   You have to configure the following communication settings.
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ip_address := "10.5.5.100";
    port_no := 9876;
    retries_connect := 2;
    timeout_connect := 4;
    retries_recv := 2;
    timeout_recv := 4;
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (2)Connect to the FH server
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    WHILE bconnected = FALSE DO
        fhconnect fh_socket,ip_address, port_no, retries_connect, timeout_connect, err_no;

        !Error check
        IF err_no <> success THEN
            GOTO exit_program;
        ENDIF
    ENDWHILE
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! 3. Example: Processing sequence of an application
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (1)Change the scene of the FH
    !    You have to select a scene No. for your application.
    !      0 : Workpiece pickking scene
    !      1 : Grasp adjustment scene
    !      2 : Workpiece placement scene
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    scene_no := 0;

    fhsample_chgscn fh_socket, retries_recv, timeout_recv, scene_no, err_no;

    !Error check
    IF err_no <> success THEN
        GOTO exit_program;
    ENDIF    
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (2)Move the robot to the imaging position
    !    You have to configure the following robot position for your application.
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    std_img_pos_x := 0;
    std_img_pos_y := 0;
    std_img_pos_z := 0;
    std_img_pos_w := 0;
    std_img_pos_p := 0;
    std_img_pos_r := 0;

    !!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! The following function drives a robot motion immediately.
    ! Confirm the settings before execution.
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    fhsample_move std_img_pos_x, std_img_pos_y, std_img_pos_z, std_img_pos_w, std_img_pos_p, std_img_pos_r, err_no;

    !Error check
    IF err_no <> success THEN
        GOTO exit_program;
    ENDIF

    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (3)Register the current position of the robot to the FH before measurement
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    fhsample_regpos fh_socket, retries_recv, timeout_recv, err_no;

    !Error check
    IF err_no <> success THEN
        GOTO exit_program;
    ENDIF

    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (4)Execute measurement and get the measurement results of the FH
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    fhsample_trig fh_socket, retries_recv, timeout_recv, param, err_no;

    !Error check
    IF err_no <> success THEN
        GOTO exit_program;
    ENDIF

    !Measurement results
    res_cmd_pos_x := param{1};
    res_cmd_pos_y := param{2};
    res_cmd_pos_z := param{3};
    res_cmd_pos_w := param{4};
    res_cmd_pos_p := param{5};
    res_cmd_pos_r := param{6};
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (5)Move the robot to the approach position
    !    You have to configure the following robot approach distance for your application.
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    std_appro_dist := 30;

    res_appro_pos_x := res_cmd_pos_x;
    res_appro_pos_y := res_cmd_pos_y;
    res_appro_pos_z := res_cmd_pos_z+std_appro_dist;
    res_appro_pos_w := res_cmd_pos_w;
    res_appro_pos_p := res_cmd_pos_p;
    res_appro_pos_r := res_cmd_pos_r;

    !!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! The following function drives a robot motion immediately.
    ! Confirm the settings before execution.
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
    fhsample_move res_appro_pos_x, res_appro_pos_y, res_appro_pos_z, res_appro_pos_w, res_appro_pos_p, res_appro_pos_r,err_no;

    !Error check
    IF err_no <> success THEN
        GOTO exit_program;
    ENDIF
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (6)Move the robot to the measured position
    !    You have to edit this section for your appication.
    !    e.g: Adding an end effector control, depart path motion, etc.
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    !!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! The following function drives a robot motion immediately.
    ! Confirm the settings before execution.
    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
    fhsample_move res_cmd_pos_x, res_cmd_pos_y, res_cmd_pos_z, res_cmd_pos_w, res_cmd_pos_p, res_cmd_pos_r, err_no; 

    ! Error check
    IF err_no <> success THEN
         GOTO exit_program;
    ENDIF

    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! 4. Example: Finalization sequence
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! (1)Disconnect from the FH server
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    exit_program:
    IF bconnected = TRUE THEN
        fhclose fh_socket, err_no;
    ENDIF
    
ENDPROC

! ***************************************************************************
! Title         : Setup main
! Function name : fhsetup_main()
! Parameters    :
! Return value  : 
! ***************************************************************************
PROC fhsetup_main()
        
    VAR socketdev fh_socket;            !Socket
    VAR string ip_address;              !IP address
    VAR num port_no;                    !Port No
    VAR num retries_connect;            !Connect Retry Count
    VAR num timeout_connect;            !Connect Wait Time
    VAR num retries_recv;               !Recieve Retry Count    
    VAR num timeout_recv;               !Recieve Wait Time
    VAR num err_no;                     !Error No. Return Value. Noneed to set
    
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ! Configure network parameters for your network
    !;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        
    ip_address := "10.5.5.100";
    port_no := 9876;
    retries_connect := 2;               !(times)
    timeout_connect := 4;               !(sec)
    retries_recv := 2;                  !(times)
    timeout_recv := 0;                  !No timeout   
    
    fhsetup fh_socket, ip_address, port_no, retries_connect, timeout_connect, retries_recv, timeout_recv, err_no;
ENDPROC



! ***************************************************************************
! Title         : FH Sample Change Scene
! Function name : fhsample_chgscn(fh_sokcet, retry_recv, timeout_recv, scene_no, errno)
! Parameters    : socektdev fh_socket;      Socket
!               : num       retry_recv;     Retry count
!               : num       timeout_recv;   Connect wait time (sec)
!               : num       scene_no;       Scene No.(0-127)
! Return value  : num       err_no;         Error No.
! ***************************************************************************
PROC fhsample_chgscn(VAR socketdev fh_socket, num retries_recv, num timeout_recv, num scene_no,Var num err_no)
    
    VAR string send_string{10};
    VAR string recv_string{128};
    
    !Initialize
    FOR i FROM 1 TO max_str_length DO
        recv_string{i} := "";
    ENDFOR  
    
    FOR i FROM 1 TO max_param_num + 1 DO
        send_string{i} := "";
    ENDFOR
    
    !Check Scene No
    IF(scene_no < min_scene_no) OR (scene_no > max_scene_no) THEN
        err_no := err_scene_no;
        TPWrite("ERROR:fhsample_chgscn():");
        TPWrite("Invalid Scene No:")\Num:=err_no;
        RETURN;
    ENDIF
    
    IF bconnected = TRUE THEN
        !Create Send Stirng
        send_string{1} := "SCENE";
        send_string{2} := ValToStr(scene_no);
        
        !Send Sreing
        fhsendstring fh_socket,send_string,err_no;
        
        !Error Check
        IF err_no <> success THEN
            RETURN;
        ENDIF
        
        !Recv String
        fhrecvstring fh_socket, retries_recv, timeout_recv, recv_string, err_no;
        
        !Error Check
        IF err_no <> success THEN
            RETURN;
        ENDIF
        
        !Receive String Check
        IF ((recv_string{1} = "O") AND (recv_string{2} = "K")) <> TRUE THEN
            err_no := err_trig;
            TPWrite("ERROR:fhsample_chgscn():");
            TPWrite("Scene Change Falied:")\Num:=err_no;
            RETURN;
        ENDIF
        
        err_no := success;
        
    ELSE
        err_no := err_send;
        TPWrite("ERROR:fhsample_chgscn():");
        TPWrite("No Connection:")\Num:=err_no;
    ENDIF

ENDPROC

! ***************************************************************************
! Title         : FH Sample Regist Robot Position
! Function name : fhsample_regpos(socket_no, retries_recv, timeout_recv, err_no)
! Parameters    : socketdev socket_no;          Socket
!               : num       retries_recv;       Retry count
!               : num       timeout_recv;       Connect wait time (sec)
! Return value  : num       err_no;             Error No.
! ***************************************************************************    
PROC fhsample_regpos(VAR socketdev fh_socket,num retries_recv, num timeout_recv, VAR num err_no)

    CONST num corcount := 6;
    VAR string send_string{10};
    VAR string recv_string{128};
    VAR robtarget curTarget;
    VAR num tmppos{corcount};
    VAR string system_variable{corcount};
    VAR string system_argment{2};
    
    !Initialize
    system_argment{1}:= "SYSDATA";
    system_argment{2}:= "Macro_SY";
    system_variable{1} := "RBVAR_CUR_POS_X#";
    system_variable{2} := "RBVAR_CUR_POS_Y#";
    system_variable{3} := "RBVAR_CUR_POS_Z#";
    system_variable{4} := "RBVAR_CUR_POS_W#";
    system_variable{5} := "RBVAR_CUR_POS_P#";
    system_variable{6} := "RBVAR_CUR_POS_R#";
    
    IF bconnected = TRUE THEN
        ! Get present position
        curTarget := CRobT();
        tmppos{1} := Round((curTarget.trans.x)\Dec := 3);
        tmppos{2} := Round((curTarget.trans.y)\Dec := 3);
        tmppos{3} := Round((curTarget.trans.z)\Dec := 3);
        tmppos{4} := Round((EulerZYX(\X, curTarget.rot))\Dec := 3);
        tmppos{5} := Round((EulerZYX(\Y, curTarget.rot))\Dec := 3);
        tmppos{6} := Round((EulerZYX(\Z, curTarget.rot))\Dec := 3);

        FOR i FROM 1 TO corcount DO
            !Initialize String 
            FOR t FROM 1 TO max_str_length DO
                recv_string{i} := "";
            ENDFOR  
            FOR t FROM 1 TO max_param_num + 1 DO
                send_string{i} := "";
            ENDFOR
            
            !Set Send String
            send_string{1} := system_argment{1};
            send_string{2} := system_argment{2};
            send_string{3} := system_variable{i};
            send_string{4} := ValToStr(tmppos{i});
            
            !Send String
            fhsendstring fh_socket, send_string, err_no;
            
            !Error Check
            IF err_no <> success THEN
                RETURN;
            ENDIF
            
            !Recv String
            fhrecvstring fh_socket, retries_recv, timeout_recv, recv_string,err_no;
        
            !Error Check
            IF err_no <> success THEN
                RETURN;
            ENDIF
 
            !Receive String Check
            IF ((recv_string{1} = "O") AND (recv_string{2} = "K")) <> TRUE THEN
                err_no := err_trig;
                TPWrite("ERROR:fhsample_regpos():");
                TPWrite("Trigger NG:")\Num:=err_no;
                RETURN;
            ENDIF
            
        ENDFOR  
        
        err_no := success;
    ELSE
        !Check error;
        err_no := err_send;
        TPWrite("ERROR:fhsample_regpos():");
        TPWrite("No Connection:")\Num:=err_no;      
    ENDIF

ENDPROC
    
! ***************************************************************************
! Title         : FH Sample Trigger
! Function name : fhsample_trig(fh_socket, retries_rev, timeout_recv, param{*}, VAR num err_no)
! Parameters    : socketdev fh_socket;      Socket
!               : num       retries_recv;   Retry count
!               : num       timeout_recv;   Connect wait time (sec)
! Return value  : num       param{*};       Target Position Data
!               : num       err_no;         Error No.
! ***************************************************************************   
 PROC fhsample_trig(VAR socketdev fh_socket, num retries_recv, num timeout_recv, VAR num param{*},VAR num err_no)
    
     VAR string send_string{10};   
     
    !Initialize
    FOR i FROM 1 TO max_param_num + 1 DO
        send_string{i} := "";
    ENDFOR  
     
    IF bconnected = TRUE THEN
        !Set Send String
        send_string{1} := "MEASURE";
        
        !Send String
        fhsendstring fh_socket, send_string, err_no;       
        
        !Error Check
        IF err_no <> success THEN
            RETURN;
        ENDIF
        
        !Response Receive
        fhmeasureresult retries_recv, timeout_recv, fh_socket, param, err_no;
        
        !Error Check
        IF err_no <> success THEN
            RETURN;
        ENDIF
        
        err_no := success;
    ELSE    
        !Check error;
        err_no := err_send;
        TPWrite("ERROR:fhsample_regpos():");
        TPWrite("No Connection:")\Num:=err_no;  
    ENDIF
    
 ENDPROC   
    
! ***************************************************************************
! Title         : FH Sample Move Robot
! Function name : fhsample_move(pos_x, pos_y, pos_z, pos_w, pos_p, pos_r,VAR num err_no)
! Parameters    : num      pos_x;          X
!               : num      pos_y;          Y
!               : num      pos_z;          Z
!               : num      pos_w;          yaw
!               : num      pos_p;          pitch
!               : num      pos_r;          roll
! Return value  : num      err_no;         Error No.
! ***************************************************************************  
PROC fhsample_move(num pos_x, num pos_y, num pos_z, num pos_w, num pos_p, num pos_r, VAR num err_no)

    VAR robtarget desttarget;
    VAR jointtarget jointpos;
    
    ! Get Target Position
    desttarget := CRobT();
    desttarget.trans.x := pos_x;
    desttarget.trans.y := pos_y;
    desttarget.trans.z := pos_z;
    desttarget.rot := OrientZYX(pos_r,pos_p,pos_w);

    jointpos := CalcJointT(desttarget, tool0, \WObj:=wobj0);
    
    ConfJ \On;
    WaitUntil \InPos, TRUE;
    MoveJ desttarget, tmp_speed, fine, tool0, \WObj:= wobj0;
    
    err_no := success;
    
    !Check Error 
    ERROR
    IF ERRNO = ERR_OUTSIDE_REACH THEN
        err_no := err_rangeout;
        TPWrite ("ERROR:fhsample_move():");
        TPwrite ("The Robot Position is Out of Range:")\Num:= err_no;
        RETURN;
    ENDIF
    
ENDPROC   
 
ENDMODULE