' **********************************************************************
'              (C) Copyright OMRON CORPORATION 2019             
'                      All Rights Reserved           
' **********************************************************************
' File name    : EPSON_FHRobotLib.prg       
' Description  : FH robot vision library
' **********************************************************************

#define MAX_INDEX_PARAM_ARRAY 9

'exception error
#define EXCEPTION_RECV_TIMEOUT 2910			'TCP/IP Receive timeout
#define EXCEPTION_MOVE_PULSE_OUT 4001		'Pulse out of range
#define EXCEPTION_MOVE_POSITION_OUT 4007	'Position out of range

' ***************************************************************************
' -----Global Parameters-----
' ***************************************************************************

'Connection state to the server
Global Boolean bconnected

'Tcp
Global Int32 max_retry_count
Global Int32 min_retry_count
Global Int32 max_time_out
Global Int32 min_time_out
Global Int32 max_socket_no
Global Int32 min_socket_no

'Scene No.
Global Int32 min_scene_no
Global Int32 max_scene_no

'String
Global Boolean mod_flg
Global Int32 sendrecv_mod
Global String str_robot_cmd$
Global String str_separate$
Global String str_delimiter$
Global Int32 max_param_length
Global Int32 max_str_length

'CommandID
Global Int32 cmdid_echo
Global Int32 cmdid_set_para
Global Int32 cmdid_get_para
Global Int32 cmdid_cur_pos
Global Int32 cmdid_get_ver
Global Int32 cmdid_abs_move

'Version
Global String version$

'Status
Global Int32 success
Global Int32 err_scene_no
Global Int32 err_con_input
Global Int32 err_con_retry
Global Int32 err_no_connect
Global Int32 err_con_timeout
Global Int32 err_recv_input
Global Int32 err_recv
Global Int32 err_recv_maxstr
Global Int32 err_recv_retry
Global Int32 err_recv_timeout
Global Int32 err_overlimit
Global Int32 err_cmd_id
Global Int32 err_cmd_format
Global Int32 err_rangeout
Global Int32 err_cmd_length
Global Int32 err_send
Global Int32 err_send_maxstr
Global Int32 err_close
Global Int32 err_trig
Global Int32 err_tjg
Global Int32 err_send_input

' ***************************************************************************
' Title         : Define Global Parameter Setting
' Function name : fhdefglobal()
' Parameters    :
' Return value  :
' ***************************************************************************	
Function fhdefglobal()
	
	'Connection state to the server
	bconnected = False

	'Tcp
	max_retry_count = 99
    min_retry_count = 0
    max_time_out = 99
    min_time_out = 0
	max_socket_no = 216
	min_socket_no = 201

	'Scene No.
    min_scene_no = 0
    max_scene_no = 127

	'String
    mod_flg = True
    sendrecv_mod = 10000
    str_robot_cmd$ = "RCMD"
    str_separate$ = Chr$(32)
    str_delimiter$ = Chr$(13)
    max_param_length = 12
    max_str_length = 128

	'CommandID
	cmdid_echo = 1
    cmdid_set_para = 10
    cmdid_get_para = 11
    cmdid_cur_pos = 12
    cmdid_get_ver = 13
    cmdid_abs_move = 100

	'Version
    version$ = "1.00"

	'Status
	success = 0
	err_scene_no = -1000
	err_con_input = -1200
	err_con_retry = -1202
	err_no_connect = -1203
	err_con_timeout = -1204
	err_recv_input = -1300
	err_recv = -1301
	err_recv_maxstr = -1302
	err_recv_retry = -1303
	err_recv_timeout = -1304
	err_overlimit = -1500
	err_cmd_id = -1501
	err_cmd_format = -1502
	err_rangeout = -1503
	err_cmd_length = -1504
	err_send = -1601
	err_send_maxstr = -1602
	err_close = -1701
	err_trig = -1800
	err_tjg = -1801
	err_send_input = -1900

Fend

' ***************************************************************************
' Title         : Connect to the Tcp Server
' Function name : fhconnect(fh_socket, retries_connect, timeout_connect, err_no)
' Parameters    : Integer	fh_socket				Socket device
'               : Int32		retries_connect			Connect retry count
'               : Int32		timeout_connect			Connect wait time (sec)
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhconnect(ByVal fh_socket As Integer, ByVal retries_connect As Int32, ByVal timeout_connect As Int32, ByRef err_no As Int32)
	
	'Declare variable
	Int32 ROBstatus
	Int32 countretries
	
	'Initialize data
	ROBstatus = 0
	countretries = 0
	
	If bconnected = False Then
		'Check the range of input parameter
		If (retries_connect < min_retry_count) Or (retries_connect > max_retry_count) Then
            err_no = err_con_input
            Print "ERROR:fhconnect():RETRY:" + Str$(err_no)
            Exit Function
		EndIf

		If (timeout_connect < min_time_out) Or (timeout_connect > max_time_out) Then
            err_no = err_con_input
            Print "ERROR:fhconnect():TIMEOUT:" + Str$(err_no)
            Exit Function
        EndIf

		If (fh_socket < min_socket_no) Or (fh_socket > max_socket_no) Then
            err_no = err_con_input
            Print "ERROR:fhconnect():SOCKET:" + Str$(err_no)
            Exit Function
        EndIf

label_connect:
		'Connect to Tcp server
		OpenNet #fh_socket As Client
		WaitNet #fh_socket, timeout_connect
		
		'Check status
		ROBstatus = ChkNet(fh_socket)
		
		If ROBstatus < 0 Then
			If timeout_connect = 0 Then
				Wait 0.5
				GoTo label_connect
			EndIf
			
			err_no = err_con_timeout
			Print "ERROR:fhconnect():TIMEOUT:" + Str$(err_no)
			
			If countretries < retries_connect Then
				countretries = countretries + 1
				GoTo label_connect
			Else
				err_no = err_con_retry
				Print "ERROR:fhconnect():RETRY:" + Str$(err_no)
			EndIf
		Else
			bconnected = True
			err_no = success
		EndIf
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Socket Close
' Function name : fhclose(fh_socket, err_no)
' Parameters    : Integer	fh_socket				Socket device
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhclose(ByVal fh_socket As Integer, ByRef err_no As Int32)
	
	'Declare variable
	Int32 ROBstatus
	
	'Initialize data
	ROBstatus = 0
	
	If bconnected = True Then
		CloseNet #fh_socket
		bconnected = False
		
		'Check status
		ROBstatus = ChkNet(fh_socket)
		
		If ROBstatus <> -3 Then
			err_no = err_close
			Print "ERROR:fhclose():CLOSE:" + Str$(err_no)
			Exit Function
		EndIf
		
		err_no = success
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Receive String from Tcp Server
' Function name : fhrecvstring(fh_socket, retries_recv, recv_string$, err_no)
' Parameters    : Integer	fh_socket				Socket device
'                 Int32		retries_recv			Receive retry count
' Return value  : String    recv_string$			Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhrecvstring(ByVal fh_socket As Integer, ByVal retries_recv As Int32, ByRef recv_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	Int32 ROBstatus
	Int32 countretries
	Int32 str_len
	
	'Initialize data
	ROBstatus = 0
	countretries = 0
	str_len = 0
	recv_string$ = ""
	
	'Check the range of input parameter
	If (retries_recv < min_retry_count) Or (retries_recv > max_retry_count) Then
        err_no = err_recv_input
        Print "ERROR:fhrecvstring():RETRY:" + Str$(err_no)
        Exit Function
    EndIf
	
	If (fh_socket < min_socket_no) Or (fh_socket > max_socket_no) Then
        err_no = err_recv_input
        Print "ERROR:fhrecvstring():SOCKET:" + Str$(err_no)
        Exit Function
    EndIf
	
	If bconnected = True Then
label_receive_string:
		OnErr GoTo label_err_recv
		Line Input #fh_socket, recv_string$
		OnErr GoTo 0
		
		If (Len(recv_string$) = 0) And (err_no = err_recv_timeout) Then
	        If countretries < retries_recv Then
				countretries = countretries + 1
				Wait 0.5
	            GoTo label_receive_string
			Else
				err_no = err_recv_retry
				Print "ERROR:fhrecvstring():RETRY:" + Str$(err_no)
				Exit Function
			EndIf
		EndIf
		
		'Check string length
		str_len = Len(recv_string$)

		If str_len = 0 Then
			err_no = err_recv
			Print "ERROR:fhrecvstring():NO_DATA:" + Str$(err_no)
			Exit Function
		ElseIf str_len > (max_str_length - 1) Then
			recv_string$ = ""
			err_no = err_recv_maxstr
			Print "ERROR:fhrecvstring():STRING_LEN:" + Str$(err_no)
			Exit Function
		EndIf
		
		err_no = success
	Else
		err_no = err_recv
		Print "ERROR:fhrecvstring():NO_CONNECT:" + Str$(err_no)
		Exit Function
	EndIf
	
	Exit Function
	
label_err_recv:
	If (Err = EXCEPTION_RECV_TIMEOUT) Then
		err_no = err_recv_timeout
		Print "ERROR:fhrecvstring():TIMEOUT:" + Str$(err_no)
		EResume Next
	Else
		OnErr GoTo 0
		EResume
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Send String to Tcp Server
' Function name : fhsendstring(fh_socket, send_string$, err_no)
' Parameters    : Integer	fh_socket				Socket device
'                 String    send_string$			Response string
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhsendstring(ByVal fh_socket As Integer, ByVal send_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	Int32 ROBstatus
	Int32 str_len
	
	'Initialize data
	ROBstatus = 0
	str_len = 0
	
	'Check the range of input parameter
	If (fh_socket < min_socket_no) Or (fh_socket > max_socket_no) Then
        err_no = err_send_input
        Print "ERROR:fhsendstring():SOCKET:" + Str$(err_no)
        Exit Function
    EndIf

	'Check string length
	str_len = Len(send_string$)
	
	If str_len = 0 Then
		err_no = err_send
		Print "ERROR:fhsendstring():NO_DATA:" + Str$(err_no)
		Exit Function
	ElseIf str_len >= max_str_length Then
		err_no = err_send_maxstr
		Print "ERROR:fhsendstring():STRING_LEN:" + Str$(err_no)
		Exit Function
	EndIf
	
	'Check status
	ROBstatus = ChkNet(fh_socket)
	
	If ROBstatus < 0 Then
		err_no = err_send
		Print "ERROR:fhsendstring():NO_CONNECT:" + Str$(err_no)
		Exit Function
	EndIf
	
	'Send string
	If bconnected = True Then
		Print #fh_socket, send_string$
		err_no = success
		Exit Function
	Else
		err_no = err_send
		Print "ERROR:fhsendstring():NO_CONNECT:" + Str$(err_no)
		Exit Function
	EndIf

	Exit Function
	
Fend

' ***************************************************************************
' Title         : Decode Command String
' Function name : fhdecode(command_string$, cmdparam(), paracount, err_no) 
' Parameters    : String    command_string$			Command string
' Return value  : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhdecode(ByVal command_string$ As String, ByRef cmdparam() As Double, ByRef paracount As Int32, ByRef err_no As Int32)
	
	'Declare variable
	Int32 loop_i
	Int32 cmdstr_len
	String cmdsplit$(0)
	String str_header$
	Int32 tmp_part_cmd_len
	Int32 dpos
	
	'Initialize data
	loop_i = 0
	cmdstr_len = 0
	cmdsplit$(0) = ""
	str_header$ = ""
	tmp_part_cmd_len = 0
	dpos = 0
	
	For loop_i = 0 To UBound(cmdparam) Step 1
		cmdparam(loop_i) = 0
	Next
	
	paracount = 0
	
	'Check string length
	cmdstr_len = Len(command_string$)
	
	If cmdstr_len <= 0 Then
		err_no = err_cmd_length
		Print "ERROR:fhdecode():NO_CMD:" + Str$(err_no)
		Exit Function
	EndIf
	
	'Split command string
	ParseStr command_string$, cmdsplit$(), str_separate$
	
	'Check command header
	str_header$ = cmdsplit$(0)
	If str_header$ <> str_robot_cmd$ Then
		err_no = err_cmd_format
		Print "ERROR:fhdecode():NO_HEADER:" + Str$(err_no)
		Exit Function
	EndIf
	
	'Get parameter
	For loop_i = 1 To UBound(cmdsplit$)
		tmp_part_cmd_len = Len(cmdsplit$(loop_i))

		If tmp_part_cmd_len > max_param_length Then
			err_no = err_cmd_format
			Print "ERROR:fhdecode():PARAM_TOO_LONG:" + Str$(err_no)
			Exit Function
		EndIf
		
		'xxxx.xxxx -> xxxx.xxx
		dpos = InStr(cmdsplit$(loop_i), ".")
		If dpos <> -1 Then
			If (tmp_part_cmd_len - dpos) >= 4 Then
				cmdsplit$(loop_i) = Mid$(cmdsplit$(loop_i), 1, dpos + 3)
			EndIf
		EndIf

		cmdparam(loop_i - 1) = Val(cmdsplit$(loop_i))
		
		paracount = paracount + 1
		If paracount = MAX_INDEX_PARAM_ARRAY + 1 Then
			GoTo label_end_get_parameter
		EndIf
	Next loop_i
	
label_end_get_parameter:
	'Check Param Num
	If paracount < 1 Then
		err_no = err_cmd_format
		Print "ERROR:fhdecode():PARAM_NUM:" + Str$(err_no)
		Exit Function
	EndIf
	
	err_no = success
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Command Implement
' Function name : fhsetup(fh_socket, retries_connect, timeout_connect, retries_recv, err_no)
' Parameters    : Integer	fh_socket				Socket device
'                 Int32		retries_connect		    Connect retry count
'                 Int32		timeout_connect		    Connect wait time (sec)
'                 Int32		retries_recv			Receive retry count
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhsetup(ByVal fh_socket As Integer, ByVal retries_connect As Int32, ByVal timeout_connect As Int32, ByVal retries_recv As Int32, ByRef err_no As Int32)
	
	'Declare variable
	String recv_string$
	Double cmdparam(MAX_INDEX_PARAM_ARRAY)
	Int32 paracount
	Int32 cmd_id
	String res_string$
	String send_string$
	Int32 loop_i
	
	'Initialize data
	recv_string$ = ""
	paracount = 0
	cmd_id = 0
	res_string$ = ""
	send_string$ = ""
	loop_i = 0
	For loop_i = 0 To UBound(cmdparam) Step 1
		cmdparam(loop_i) = 0
	Next
	
	'Global definition
	Call fhdefglobal()
	
	'Check status of connect
	If bconnected = True Then
		Call fhclose(fh_socket, ByRef err_no)
	EndIf
	
	'Connect
	If bconnected = False Then
		Call fhconnect(fh_socket, retries_connect, timeout_connect, ByRef err_no)
		If err_no <> success Then
        	GoTo label_exit_func
        EndIf
	EndIf
	
	Do While bconnected = True
		'Initialize data
		paracount = 0
		recv_string$ = ""
		
		'Receive string
		Call fhrecvstring(fh_socket, retries_recv, ByRef recv_string$, ByRef err_no)
		
		'Error check
		If err_no <> success Then
			GoTo label_exit_func
		EndIf
		
		'Decode string
		Call fhdecode(recv_string$, ByRef cmdparam(), ByRef paracount, ByRef err_no)
		
		'Error check
		If err_no <> success Then
			GoTo label_exit_func
		EndIf
		
		If paracount > 0 Then
			cmd_id = cmdparam(0)
			Select cmd_id
				Case cmdid_echo
					Call fhecho(ByRef cmdparam(), paracount, ByRef res_string$, ByRef err_no)
				Case cmdid_set_para
					Call fhsetrobotpara(ByRef cmdparam(), paracount, ByRef res_string$, ByRef err_no)
				Case cmdid_get_para
					Call fhgetrobotpara(ByRef cmdparam(), paracount, ByRef res_string$, ByRef err_no)
				Case cmdid_cur_pos
					Call fhgetcurpos(ByRef cmdparam(), paracount, mod_flg, ByRef res_string$, ByRef err_no)
				Case cmdid_get_ver
					Call fhgetverinfo(ByRef cmdparam(), paracount, ByRef res_string$, ByRef err_no)
				Case cmdid_abs_move
					Call fhabsmove(ByRef cmdparam(), paracount, mod_flg, ByRef res_string$, ByRef err_no)
				Default
					err_no = err_cmd_id
					Print "ERROR:fhsetup():UNDEFINED_ID:" + Str$(err_no)
					res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmd_id) + str_separate$ + Str$(err_no)
			Send
			
			'Send response string
			send_string$ = res_string$
			Call fhsendstring(fh_socket, send_string$, ByRef err_no)
			
			'Error check
			If err_no <> success Then
				GoTo label_exit_func
			EndIf
		EndIf
	Loop
	
label_exit_func:
	If err_no <> success Then
		Call fhclose(fh_socket, ByRef err_no)
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Command Echo
' Function name : fhecho(cmdparam(), paracount, echo_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
' Return value  : String	echo_string$			Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhecho(ByRef cmdparam() As Double, ByVal paracount As Int32, ByRef echo_string$ As String, ByRef err_no As Int32)
	
	'Check parameter count
	If (cmdparam(0) = cmdid_echo) And (paracount = 1) Then
        err_no = success
    Else
    	err_no = err_cmd_format
    	Print "ERROR:fhecho():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set response string
	echo_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no)
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Set Robot Parameter
' Function name : fhsetrobotpara(cmdparam(), paracount, res_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
' Return value  : String	res_string$				Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhsetrobotpara(ByRef cmdparam() As Double, ByVal paracount As Int32, ByRef res_string$ As String, ByRef err_no As Int32)
	
	'Declare variable	
	Double param_speed
	
	'Initialize data
	param_speed = 0
	
	'Check parameter count
	If (cmdparam(0) = cmdid_set_para) And (paracount = 2) Then
		param_speed = cmdparam(1)
		
		'Check speed
		If (param_speed < 1) Or (param_speed > 100) Then
			err_no = err_overlimit
			Print "ERROR:fhsetrobotpara():SPEED:" + Str$(err_no)
		EndIf
		
		'Set robot parameters
		If err_no = success Then
			Speed Int(param_speed + 0.5)
			err_no = success
		EndIf
	Else
		err_no = err_cmd_format
		Print "ERROR:fhsetrobotpara():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set response string
	res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no)
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Get Robot Parameter
' Function name : fhgetrobotpara(cmdparam(), paracount, res_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
' Return value  : String	res_string$				Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhgetrobotpara(ByRef cmdparam() As Double, ByVal paracount As Int32, ByRef res_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	Double param_speed
	
	'Initialize data
	param_speed = 0
	
	'Check parameter count
	If (cmdparam(0) = cmdid_get_para) And (paracount = 1) Then
		' Get parameter
		param_speed = Speed(1)
		err_no = success
	Else
		param_speed = 0
		err_no = err_cmd_format
		Print "ERROR:fhgetrobotpara():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set response string
	res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no) + str_separate$ + Str$(Int(param_speed + 0.5))
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Get Current Position
' Function name : fhgetcurpos(cmdparam(), paracount, intflg ,res_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
'                 Int32		intflg					Param choice 0:Real 1:Real*10000
' Return value  : String	res_string$				Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhgetcurpos(ByRef cmdparam() As Double, ByVal paracount As Int32, ByVal intflg As Boolean, ByRef res_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	Int32 param_local_no
	Int32 param_tool_no
	Double tmp_posX
	Double tmp_posY
	Double tmp_posZ
	Double tmp_posW
	Double tmp_posP
	Double tmp_posR
	Double pospara(5)
	Int32 loop_i
	String tmp_str$
	String locstr$
	Int32 corcount
	
	'Initialize data
	param_local_no = 0
	param_tool_no = 0
	tmp_posX = 0
	tmp_posY = 0
	tmp_posZ = 0
	tmp_posW = 0
	tmp_posP = 0
	tmp_posR = 0
	loop_i = 0
	tmp_str$ = ""
	locstr$ = ""
	corcount = 5
	For loop_i = 0 To UBound(pospara) Step 1
		pospara(loop_i) = 0
	Next
	
	'Check parameter count
	If (cmdparam(0) = cmdid_cur_pos) And (paracount = 1) Then
		'Change coordinate system
		Tool param_tool_no
		
		'Get current position
		tmp_posX = CX(Here @(param_local_no))
		tmp_posY = CY(Here @(param_local_no))
		tmp_posZ = CZ(Here @(param_local_no))
		tmp_posW = CW(Here @(param_local_no))
		tmp_posP = CV(Here @(param_local_no))
		tmp_posR = CU(Here @(param_local_no))
		
		If intflg = True Then
			pospara(0) = tmp_posX * sendrecv_mod
			pospara(1) = tmp_posY * sendrecv_mod
			pospara(2) = tmp_posZ * sendrecv_mod
			pospara(3) = tmp_posW * sendrecv_mod
			pospara(4) = tmp_posP * sendrecv_mod
			pospara(5) = tmp_posR * sendrecv_mod
		Else
			pospara(0) = tmp_posX
			pospara(1) = tmp_posY
			pospara(2) = tmp_posZ
			pospara(3) = tmp_posW
			pospara(4) = tmp_posP
			pospara(5) = tmp_posR
		EndIf
		
		err_no = success
	Else
		err_no = err_cmd_format
		Print "ERROR:fhgetcurpos():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set response string
	If err_no <> success Then
        For loop_i = 0 To corcount Step 1
            locstr$ = locstr$ + str_separate$ + Str$(pospara(loop_i))
        Next
    Else
    	For loop_i = 0 To corcount Step 1
    		If intflg = 1 Then
				pospara(loop_i) = Int(pospara(loop_i) + 0.5)
				tmp_str$ = Str$(pospara(loop_i))
			Else
				'Ignore 4th decimal part
				'xxxx.xxxx -> xxxx.xxx
				tmp_str$ = FmtStr$(pospara(loop_i), ".000")
			EndIf
			locstr$ = locstr$ + str_separate$ + tmp_str$
    	Next
    EndIf
	
	res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no) + locstr$
    
    Exit Function
Fend

' ***************************************************************************
' Title         : Get Version Information
' Function name : fhgetverinfo(cmdparam(), paramcount, res_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
' Return value  : String	res_string$				Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhgetverinfo(ByRef cmdparam() As Double, ByVal paracount As Int32, ByRef res_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	String local_ver$
	
	'Initialize data
	local_ver$ = ""
	
	'Check parameter count
	If (cmdparam(0) = cmdid_get_ver) And (paracount = 1) Then
		local_ver$ = version$
		err_no = success
	Else
		local_ver$ = "0.00"
		err_no = err_cmd_format
		Print "ERROR:fhgetverinfo():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set response string
	res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no) + str_separate$ + local_ver$
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Move to Specified Position
' Function name : fhabsmove(cmdparam(), paracount, intflg, res_string$, err_no)
' Parameters    : Double	cmdparam()				Command parameter
'                 Int32		paracount				Parameter count
'                 Int32		intflg					Param choice 0:Real 1:Real*10000
' Return value  : String	res_string$				Response string
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhabsmove(ByRef cmdparam() As Double, ByVal paracount As Int32, ByVal intflg As Boolean, ByRef res_string$ As String, ByRef err_no As Int32)
	
	'Declare variable
	Int32 param_local_no
	Int32 param_tool_no
	Int32 loop_i
	Double end_pos(5)
	Double x
	Double y
	Double z
	Double w
	Double p
	Double r
	Double tmp_posX
	Double tmp_posY
	Double tmp_posZ
	Double tmp_posW
	Double tmp_posP
	Double tmp_posR
	Double pospara(5)
	String tmp_str$
	String locstr$
	Int32 corcount
	
	'Initialize data
	param_local_no = 0
	param_tool_no = 0
	loop_i = 0
	x = 0
	y = 0
	z = 0
	w = 0
	p = 0
	r = 0
	tmp_posX = 0
	tmp_posY = 0
	tmp_posZ = 0
	tmp_posW = 0
	tmp_posP = 0
	tmp_posR = 0
	tmp_str$ = ""
	locstr$ = ""
	corcount = 5
	For loop_i = 0 To UBound(end_pos) Step 1
		end_pos(loop_i) = 0
	Next
	For loop_i = 0 To UBound(pospara) Step 1
		pospara(loop_i) = 0
	Next
	
	'Check parameter count
	If (cmdparam(0) = cmdid_abs_move) And (paracount = 7) Then
		'Change coordinate system
		Tool param_tool_no
		
		'Get specified position
		For loop_i = 0 To corcount Step 1
			end_pos(loop_i) = cmdparam(loop_i + 1)
		Next
		
		If intflg = True Then
			x = end_pos(0) / sendrecv_mod
			y = end_pos(1) / sendrecv_mod
			z = end_pos(2) / sendrecv_mod
			w = end_pos(3) / sendrecv_mod
			p = end_pos(4) / sendrecv_mod
			r = end_pos(5) / sendrecv_mod
		Else
			x = end_pos(0)
			y = end_pos(1)
			z = end_pos(2)
			w = end_pos(3)
			p = end_pos(4)
			r = end_pos(5)
		EndIf
		
		'Move
		OnErr GoTo label_err_move
		Go XY(x, y, z, r, p, w) /(param_local_no)
		OnErr GoTo 0
		err_no = success
		
		'Get current position
		tmp_posX = CX(Here @(param_local_no))
		tmp_posY = CY(Here @(param_local_no))
		tmp_posZ = CZ(Here @(param_local_no))
		tmp_posW = CW(Here @(param_local_no))
		tmp_posP = CV(Here @(param_local_no))
		tmp_posR = CU(Here @(param_local_no))
		
		If intflg = True Then
			pospara(0) = tmp_posX * sendrecv_mod
			pospara(1) = tmp_posY * sendrecv_mod
			pospara(2) = tmp_posZ * sendrecv_mod
			pospara(3) = tmp_posW * sendrecv_mod
			pospara(4) = tmp_posP * sendrecv_mod
			pospara(5) = tmp_posR * sendrecv_mod
		Else
			pospara(0) = tmp_posX
			pospara(1) = tmp_posY
			pospara(2) = tmp_posZ
			pospara(3) = tmp_posW
			pospara(4) = tmp_posP
			pospara(5) = tmp_posR
		EndIf
	Else
		err_no = err_cmd_format
		Print "ERROR:fhabsmove():PARAM_NUM:" + Str$(err_no)
	EndIf
	
	'Set Response String
	If err_no <> success Then
		For loop_i = 0 To corcount Step 1
			tmp_str$ = Str$(pospara(loop_i))
			locstr$ = locstr$ + str_separate$ + tmp_str$
		Next
	Else
		For loop_i = 0 To corcount Step 1
			If intflg = True Then
				pospara(loop_i) = Int(pospara(loop_i) + 0.5)
				tmp_str$ = Str$(pospara(loop_i))
			Else
				'Ignore 4th decimal part
				'xxxx.xxxx -> xxxx.xxx
				tmp_str$ = FmtStr$(pospara(loop_i), ".000")
			EndIf
			locstr$ = locstr$ + str_separate$ + tmp_str$
		Next
    EndIf
    
	res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no) + locstr$
	
	Exit Function

label_err_move:
	If (Err = EXCEPTION_MOVE_PULSE_OUT) Or (Err = EXCEPTION_MOVE_POSITION_OUT) Then
		err_no = err_rangeout
		Print "ERROR:fhabsmove():OUT_RANGE:" + Str$(err_no)
		Redim pospara(corcount)
		For loop_i = 0 To corcount Step 1
			tmp_str$ = Str$(pospara(loop_i))
			locstr$ = locstr$ + str_separate$ + tmp_str$
		Next
		res_string$ = str_robot_cmd$ + str_separate$ + Str$(cmdparam(0)) + str_separate$ + Str$(err_no) + locstr$
		Exit Function
	Else
		OnErr GoTo 0
			EResume
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : Get Measured Position
' Function name : fhmeasureresult(retries_recv, fh_socket, param(), err_no)
' Parameters    : Int32		retries_recv			Receive retry count
'               : Integer	fh_socket				Socket_no
' Return value  : Double	param					Measured position
'                 Int32		err_no					Error No.
' ***************************************************************************
Function fhmeasureresult(ByVal retries_recv As Int32, ByVal fh_socket As Integer, ByRef param() As Double, ByRef err_no As Int32)
	
	'Declare variable
	String recv_string$
	Double cmdparam(MAX_INDEX_PARAM_ARRAY)
	Int32 paracount
	Int32 loop_i
	
	'Initialize data
	recv_string$ = ""
	paracount = 0
	loop_i = 0
	For loop_i = 0 To UBound(cmdparam) Step 1
		cmdparam(loop_i) = 0
	Next
	For loop_i = 0 To UBound(param) Step 1
		param(loop_i) = 0
	Next
	
	If bconnected = True Then
		'Receive string
		Call fhrecvstring(fh_socket, retries_recv, ByRef recv_string$, ByRef err_no)
		
		'Error check
		If err_no <> success Then
			Exit Function
		EndIf
		
		'Receive string check
		If recv_string$ <> "OK" Then
			err_no = err_trig
			Print "ERROR:fhmeasureresult():Trigger NG:" + Str$(err_no)
			Exit Function
		EndIf
		
		'Receive string
		recv_string$ = ""
		Call fhrecvstring(fh_socket, retries_recv, ByRef recv_string$, ByRef err_no)
		
		'Error check
		If err_no <> success Then
			Exit Function
		EndIf
		
		'Add command header
		recv_string$ = str_robot_cmd$ + str_separate$ + recv_string$
		
		'Decode string
		Call fhdecode(recv_string$, ByRef cmdparam(), ByRef paracount, ByRef err_no)
		
		'Error check
		If err_no <> success Then
			Exit Function
		EndIf
		
		'Format check
		If paracount <> 7 Then
			err_no = err_cmd_format
			Print "ERROR:fhmeasureresult():Invalid Parameter:" + Str$(err_no)
			Exit Function
		EndIf
		
		'Total judge check
		If cmdparam(0) <> 1 Then
			err_no = err_tjg
			Print "ERROR:fhmeasureresult():Total Judge NG:" + Str$(err_no)
			Exit Function
		EndIf
		
		'Parameter setting
		For loop_i = 0 To paracount - 2
			param(loop_i) = cmdparam(loop_i + 1)
		Next
		
		err_no = success
	Else
		err_no = err_no_connect
		Print "ERROR:fhmeasureresult():No Connection:" + Str$(err_no)
		Exit Function
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : FH Sample Change Scene
' Function name : fhsample_chgscn(fh_sokcet, retries_recv, scene_no, err_no)
' Parameters    : Integer	fh_socket				Socket
'               : Int32		retries_recv			Receive retry count
'               : Int32		scene_no				Scene No.(0-127)
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhsample_chgscn(ByVal fh_socket As Integer, ByVal retries_recv As Int32, ByVal scene_no As Integer, ByRef err_no As Int32)
	
	'Declare variable
	String send_string$
	String res_string$
	
	'Initialize data
	send_string$ = ""
	res_string$ = ""
	
	'Check Scene No
	If (scene_no < min_scene_no) Or (scene_no > max_scene_no) Then
		err_no = err_scene_no
		Print "ERROR:fhsample_chgscn():Invalid Scene No.:" + Str$(err_no)
		Exit Function
	EndIf
	
	If bconnected = True Then
		send_string$ = "SCENE " + Str$(scene_no)
		
		'Send command string
		Call fhsendstring(fh_socket, send_string$, ByRef err_no)
		
		'Error Check
        If err_no <> success Then
            Exit Function
        EndIf
        
        'Receive string
		Call fhrecvstring(fh_socket, retries_recv, ByRef res_string$, ByRef err_no)
		
		'Error Check
        If err_no <> success Then
            Exit Function
        EndIf
        
        'Receive string check
		If res_string$ <> "OK" Then
			err_no = err_trig
			Print "ERROR:fhsample_chgscn():Scene Change Failed:" + Str$(err_no)
			Exit Function
		EndIf
		
		err_no = success
	Else
		err_no = err_send
		Print "ERROR:fhsample_chgscn():NO Connection:" + Str$(err_no)
		Exit Function
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : FH Sample Regist Robot Position
' Function name : fhsample_regpos(socket_no, retries_recv, err_no)
' Parameters    : Integer	fh_socket				Socket
'               : Int32		retries_recv			Receive retry count
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhsample_regpos(ByVal fh_socket As Integer, ByVal retries_recv As Int32, ByRef err_no As Int32)
	
	'Declare variable
	String system_argment$
	String system_variable$(5)
	Int32 param_local_no
	Int32 param_tool_no
	String send_string$
	String res_string$
	String tmp_str$
	Double tmp_posX
	Double tmp_posY
	Double tmp_posZ
	Double tmp_posW
	Double tmp_posP
	Double tmp_posR
	Double tmp_pos(5)
	Int32 loop_i
	Int32 corcount
	
	'Initialize data
	system_argment$ = "SYSDATA Macro_SY"
	system_variable$(0) = "RBVAR_CUR_POS_X#"
	system_variable$(1) = "RBVAR_CUR_POS_Y#"
	system_variable$(2) = "RBVAR_CUR_POS_Z#"
	system_variable$(3) = "RBVAR_CUR_POS_W#"
	system_variable$(4) = "RBVAR_CUR_POS_P#"
	system_variable$(5) = "RBVAR_CUR_POS_R#"
	param_local_no = 0
	param_tool_no = 0
	send_string$ = ""
	res_string$ = ""
	tmp_str$ = ""
	tmp_posX = 0
	tmp_posY = 0
	tmp_posZ = 0
	tmp_posW = 0
	tmp_posP = 0
	tmp_posR = 0
	loop_i = 0
	corcount = 5
	For loop_i = 0 To UBound(tmp_pos) Step 1
		tmp_pos(loop_i) = 0
	Next

	'Change coordinate system
	Tool param_tool_no
	
	If bconnected = True Then
		'Get current position
		tmp_posX = CX(Here @(param_local_no))
		tmp_posY = CY(Here @(param_local_no))
		tmp_posZ = CZ(Here @(param_local_no))
		tmp_posW = CW(Here @(param_local_no))
		tmp_posP = CV(Here @(param_local_no))
		tmp_posR = CU(Here @(param_local_no))
		
		tmp_pos(0) = tmp_posX
		tmp_pos(1) = tmp_posY
		tmp_pos(2) = tmp_posZ
		tmp_pos(3) = tmp_posW
		tmp_pos(4) = tmp_posP
		tmp_pos(5) = tmp_posR
		
		For loop_i = 0 To corcount Step 1
			'Initialize data
			send_string$ = ""
			res_string$ = ""
			
			'Make command string
			tmp_str$ = FmtStr$(tmp_pos(loop_i), ".000")
			send_string$ = system_argment$ + str_separate$ + system_variable$(loop_i) + str_separate$ + tmp_str$
			
			'Send command string
			Call fhsendstring(fh_socket, send_string$, ByRef err_no)
			
			'Error Check
	        If err_no <> success Then
	            Exit Function
	        EndIf
	        
			'Receive string
			Call fhrecvstring(fh_socket, retries_recv, ByRef res_string$, ByRef err_no)
			
			'Error Check
	        If err_no <> success Then
	            Exit Function
	        EndIf
	        
	        'Receive string check
			If res_string$ <> "OK" Then
                err_no = err_trig
                Print "ERROR:fhsample_regpos():Trigger NG:" + Str$(err_no)
                Exit Function
            EndIf
		Next
		
		err_no = success
	Else
		err_no = err_send
		Print "ERROR:fhsample_regpos():NO Connection:" + Str$(err_no)
		Exit Function
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : FH Sample Trigger
' Function name : fhsample_trig(fh_socket, retries_rev, param(), err_no)
' Parameters    : Integer	fh_socket				Socket
'               : Int32		retries_recv			Receive retry count
' Return value  : Double	param()					Target Position Data
'               : Int32		err_no					Error No.
' ***************************************************************************
Function fhsample_trig(ByVal fh_socket As Integer, ByVal retries_recv As Int32, ByRef param() As Double, ByRef err_no As Int32)
	
	'Declare variable	
	String send_string$
	Int32 loop_i
	
	'Initialize data
	send_string$ = ""
	loop_i = 0
	For loop_i = 0 To UBound(param) Step 1
		param(loop_i) = 0
	Next
	
	If bconnected = True Then
		'Make command string
		send_string$ = "MEASURE"
		
		'Send command string
		Call fhsendstring(fh_socket, send_string$, ByRef err_no)
		
		'Error Check
        If err_no <> success Then
            Exit Function
        EndIf
        
        'Response receive
		Call fhmeasureresult(retries_recv, fh_socket, ByRef param(), ByRef err_no)
		
		'Error Check
        If err_no <> success Then
            Exit Function
        EndIf
        
        err_no = success
    Else
		err_no = err_send
		Print "ERROR:fhsample_trig():NO Connection:" + Str$(err_no)
		Exit Function
	EndIf
	
	Exit Function
Fend

' ***************************************************************************
' Title         : FH Sample Move Robot
' Function name : fhsample_move(pos_x, pos_y, pos_z, pos_w, pos_p, pos_r, err_no)
' Parameters    : Double	pos_x					X
'               : Double	pos_y					Y
'               : Double	pos_z					Z
'               : Double	pos_w					yaw
'               : Double	pos_p					pitch
'               : Double	pos_r					roll
' Return value  : Int32		err_no					Error No.
' ***************************************************************************
Function fhsample_move(ByVal pos_x As Double, ByVal pos_y As Double, ByVal pos_z As Double, ByVal pos_w As Double, ByVal pos_p As Double, ByVal pos_r As Double, ByRef err_no As Int32)
	
	'Declare variable
	Int32 param_local_no
	Int32 param_tool_no
	
	'Initialize data
	param_local_no = 0
	param_tool_no = 0
	
	'Change coordinate system
	Tool param_tool_no
	
	'Move
	OnErr GoTo label_err_move
	Go XY(pos_x, pos_y, pos_z, pos_r, pos_p, pos_w) /(param_local_no)
	OnErr GoTo 0
	err_no = success
	
	Exit Function
	
label_err_move:
	If (Err = EXCEPTION_MOVE_PULSE_OUT) Or (Err = EXCEPTION_MOVE_POSITION_OUT) Then
		err_no = err_rangeout
		Print "ERROR:fhsample_move():The Robot Position is out of range:" + Str$(err_no)
		Exit Function
	Else
		OnErr GoTo 0
		EResume
	EndIf
	
	Exit Function
Fend


