' **********************************************************************
'              (C) Copyright OMRON CORPORATION 2019             
'                      All Rights Reserved           
' **********************************************************************
' File name    : EPSON_FHRobotSample.prg       
' Description  : FH robot vision Sample
' **********************************************************************

#define MAX_INDEX_PARAM_ARRAY 9

' ***************************************************************************
' Title         : Main
' Function name : Main
' Parameters    :
' Return value  :
' ***************************************************************************
Function main
	
	Call fhsetup_main()
	
Fend

' ***************************************************************************
' Title         : Setup main
' Function name : fhsetup_main()
' Parameters    :
' Return value  : 
' ***************************************************************************
Function fhsetup_main()
	
	'Initialize data
	Integer fh_socket			'Socket
	Int32 retries_connect		'Connect Retry Count	
    Int32 timeout_connect		'Connect Wait Time
    Int32 retries_recv			'Recieve Retry Count    
    Int32 err_no				'Error No. Return Value. Noneed to set
    
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ' Configure network parameters for your network
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    fh_socket = 201
    retries_connect = 2					'(times)
    timeout_connect = 4					'(sec)
    retries_recv = 2					'(times)
    
    Call fhsetup(fh_socket, retries_connect, timeout_connect, retries_recv, ByRef err_no)
	
Fend

' ***************************************************************************
' Title         : FH Sample Main
' Function name : fhsample_main()
' Parameters    :
' Return value  :
' ***************************************************************************
Function fhsample_main()
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' 1. Example: Initialization sequence
	' (1) Declare local variables
	' (2) Initialize global variables
	'
	' 2. Example: Network connection sequence
	' (1) Set the network configuration
	' (2) Connect to the FH server
	'
	' 3. Example: Processing sequence of an application
	' (1) Change the scene of the FH
	' (2) Move the robot to the imaging position
	' (3) Register the current position of the robot to the FH before measurement
	' (4) Execute measurement and get the measurement results of the FH
	' (5) Move the robot to the approach position
	' (6) Move the robot to the measured position
	'
	' 4. Example: Finalization sequence
	' (1) Disconnect to the FH server
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ' 1. Example: Initialization sequence
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ' (1)Declare local variables
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Integer fh_socket			'COM Line No.
	Int32 retries_connect		'Connect Retry Count
	Int32 timeout_connect		'Connect Wait Time
	Int32 retries_recv			'Recieve Retry Count    
	Int32 err_no				'Error No. Return Value. Noneed to set

	Int32 scene_no

	Double std_img_pos_x
	Double std_img_pos_y
	Double std_img_pos_z
	Double std_img_pos_w
	Double std_img_pos_p
	Double std_img_pos_r

	Double res_cmd_pos_x
	Double res_cmd_pos_y
	Double res_cmd_pos_z
	Double res_cmd_pos_w
	Double res_cmd_pos_p
	Double res_cmd_pos_r

	Double res_appro_pos_x
	Double res_appro_pos_y
	Double res_appro_pos_z
	Double res_appro_pos_w
	Double res_appro_pos_p
	Double res_appro_pos_r

	Double std_appro_dist
	Double param(MAX_INDEX_PARAM_ARRAY)
    
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ' (2)Initialaize global variables
    ';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Call fhdefglobal()

	err_no = success
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' 2. Example: Network connection sequence
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (1)Set the network configuration
	'   You have to configure the following communication settings.
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	fh_socket = 201
	retries_connect = 2
	timeout_connect = 4
	retries_recv = 2
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (2)Connect to the FH server
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	Do While bconnected = 0
		Call fhconnect(fh_socket, retries_connect, timeout_connect, ByRef err_no)

		'Error check
		If err_no <> success Then
			GoTo exit_program
		EndIf
	Loop
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' 3. Example: Processing sequence of an application
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (1)Change the scene of the FH
	'    You have to select a scene No. for your application.
	'      0 : Workpiece pickking scene
	'      1 : Grasp adjustment scene
	'      2 : Workpiece placement scene
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	scene_no = 0
	
	Call fhsample_chgscn(fh_socket, retries_recv, scene_no, ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (2)Move the robot to the imaging position
	'    You have to configure the following robot position for your application.
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	std_img_pos_x = 0
	std_img_pos_y = 0
	std_img_pos_z = 0
	std_img_pos_w = 0
	std_img_pos_p = 0
	std_img_pos_r = 0
	
	'!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	' The following function drives a robot motion immediately.
	' Confirm the settings before execution.
	'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Call fhsample_move(std_img_pos_x, std_img_pos_y, std_img_pos_z, std_img_pos_w, std_img_pos_p, std_img_pos_r, ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (3)Register the current position of the robot to the FH before measurement
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	Call fhsample_regpos(fh_socket, retries_recv, ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (4)Execute measurement and get the measurement results of the FH
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	Call fhsample_trig(fh_socket, retries_recv, ByRef param(), ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf
	
	'Measurement results
	res_cmd_pos_x = param(0)
	res_cmd_pos_y = param(1)
	res_cmd_pos_z = param(2)
	res_cmd_pos_w = param(3)
	res_cmd_pos_p = param(4)
	res_cmd_pos_r = param(5)
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (5)Move the robot to the approach position
	'    You have to configure the following robot approach distance for your application.
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	std_appro_dist = 30

	res_appro_pos_x = res_cmd_pos_x
	res_appro_pos_y = res_cmd_pos_y
	res_appro_pos_z = res_cmd_pos_z + std_appro_dist
	res_appro_pos_w = res_cmd_pos_w
	res_appro_pos_p = res_cmd_pos_p
	res_appro_pos_r = res_cmd_pos_r
	
	'!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	' The following function drives a robot motion immediately.
	' Confirm the settings before execution.
	'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Call fhsample_move(res_appro_pos_x, res_appro_pos_y, res_appro_pos_z, res_appro_pos_w, res_appro_pos_p, res_appro_pos_r, ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf
	
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (6)Move the robot to the measured position
	'    You have to edit this section for your appication.
	'    e.g: Adding an end effector control, depart path motion, etc.
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	'!!!!!!!!! CAUTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	' The following function drives a robot motion immediately.
	' Confirm the settings before execution.
	'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Call fhsample_move(res_cmd_pos_x, res_cmd_pos_y, res_cmd_pos_z, res_cmd_pos_w, res_cmd_pos_p, res_cmd_pos_r, ByRef err_no)
	
	'Error check
	If err_no <> success Then
		GoTo exit_program
	EndIf

	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' 4. Example: Finalization sequence
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	' (1)Disconnect from the FH server
	';;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
exit_program:
	If bconnected = True Then
        Call fhclose(fh_socket, ByRef err_no)
    EndIf

Fend

